<?php
//
// MySQL DB functions for dataowner application
//

include '_load_config.php';


function is_admin()
{
  if (array_key_exists('admin', $_SESSION) AND $_SESSION["admin"] == True) {
    return True;
  } else {
    return False;
  }
}

function is_auditor()
{
  if (array_key_exists('audit', $_SESSION) AND $_SESSION["audit"] == True) {
    return True;
  } else {
    return False;
  }
}

function is_approver()
{
  if (array_key_exists('approver', $_SESSION) AND $_SESSION["approver"] == True) {
    return True;
  } else {
    return False;
  }
}

// Returns True if the current logged in user is not an approver
// and not an admin.
function is_user()
{
  $is_user_boolean = (!is_admin() && (!is_approver())) ;
  return $is_user_boolean;
}

function connect_mysql()
{
  $db_creds_file = getenv('DATAOWNER_REQUEST_DB_CREDENTIALS');
  if (!$db_creds_file) {
    $db_creds_file = '/etc/data-access-request/database.ini';
  }

  $db_creds_array = parse_ini_file($db_creds_file);

  $database_host     = $db_creds_array['host'];
  $database_username = $db_creds_array['username'];
  $database_password = $db_creds_array['password'];
  $database_db_name  = $db_creds_array['db_name'];

  $connection = mysqli_init();

  $connect_result = $connection->real_connect($database_host, $database_username, $database_password);

  if (!$connect_result) {
    die("Database connection error: could not connect "
      . "to $database_host with username $database_username: " . $connection->error);
  }

  if (! $connection->select_db($database_db_name)) {
    die('Database selection error: ' . $connection->error);
  }

  return $connection;
}

function get_bundle()
{
  $connection = connect_mysql();

  $sql = "
    SELECT group_name
    FROM   dataowner_attribute_groups
    ORDER BY
           group_name";
  $result = $connection->query($sql);
  if (!$result) {
    die('Query execution error: ' . mysql_error());
  }
  while ($row = $result->fetch_assoc()) {
    $group_array[] = $row;
  }
  $connection->close();
  return ($group_array);
}

function get_reviewers()
{
  $connection = connect_mysql();

  /*
  mysql> desc dataowner_reviewer;
  +-----------------+-----------------------------------------+------+-----+---------+-------+
  | Field           | Type                                    | Null | Key | Default | Extra |
  +-----------------+-----------------------------------------+------+-----+---------+-------+
  | uid             | varchar(32)                             | NO   | PRI | NULL    |       |
  | full_name       | varchar(64)                             | NO   |     | NULL    |       |
  | title           | varchar(64)                             | NO   |     | NULL    |       |
  | email           | varchar(255)                            | YES  |     | NULL    |       |
  | role            | enum('ADMINISTRATOR','APPROVE','AUDIT') | YES  |     | NULL    |       |
  | status          | enum('ACTIVE','INACTIVE')               | NO   |     | ACTIVE  |       |
  | date_last_maint | datetime                                | NO   |     | NULL    |       |
  | date_added      | datetime                                | NO   |     | NULL    |       |
  +-----------------+-----------------------------------------+------+-----+---------+-------+
  */

  $sql = "
    SELECT *
    FROM   dataowner_reviewer
    WHERE  status = 'ACTIVE' and
           role = 'APPROVE'
    ORDER BY
           full_name";

  $result = $connection->query($sql);
  if (!$result) {
    die('Query execution error: ' . mysql_error());
  }
  while ($row = $result->fetch_assoc()) {
    $reviewer_array[] = $row ;
  }
  $connection->close();
  return ($reviewer_array);
}

function get_attribute_list_from_group($group_name)
{
  $connection = connect_mysql();

  $attr_string = "";
  $sql = "
    SELECT attribute_name
    FROM   dataowner_attributes
    WHERE  group_name = '$group_name'
    ORDER BY
           attribute_name";
  $result = $connection->query($sql);
  if (!$result) {
    die('Query execution error: ' . mysql_error());
  }

  $attributes = [];
  while ($row = $result->fetch_assoc()) {
    array_push($attributes, $row["attribute_name"]);
  }
  $connection->close();

  return $attributes;
}

function get_attributes()
{
  $connection = connect_mysql();

  $sql = "
    SELECT distinct attribute_name
    FROM   dataowner_attributes
    ORDER BY
           attribute_name";

  $result = $connection->query($sql);
  if (!$result) {
    die('Query execution error: ' . mysql_error());
  }
  while ($row = $result->fetch_assoc()) {
    $attribute_array[] = $row ;
  }
  $connection->close();
  return ($attribute_array);
}

function get_attribute_groups()
{
  $connection = connect_mysql();

  $sql = "
    SELECT group_name
    FROM   dataowner_attribute_groups
    ORDER BY
           group_name";

  $result = $connection->query($sql);
  if (!$result) {
    die('Query execution error: ' . mysql_error());
  }
  while ($row = $result->fetch_assoc()) {
    $attribute_array[] = $row ;
  }
  $connection->close();
  return ($attribute_array);
}



function get_old_request_report()
{
  $connection = connect_mysql();

  $sql = "
    SELECT a.request_id,
           ifnull(a.approver_name, v.full_name) as approver,
           ifnull(a.status, '') as status,
           if(a.date_last_maint > a.date_added, a.date_last_maint, a.date_added) as date_last_maint
    FROM   dataowner_reviewer as v
           LEFT JOIN dataowner_approval as a on
             a.approver_uid = v.uid
           LEFT JOIN dataowner_request as r on
             r.request_id = a.request_id and
             a.request_id = r.request_id
    WHERE  v.role in ('APPROVE', 'ADMINISTRATOR') and
           v.status = 'ACTIVE'
    ORDER BY
           1 DESC, 2";

  $result = $connection->query($sql);
  while ($row = $result->fetch_assoc()) {
    $attr_array[] = $row ;
  }
  $connection->close();
  return ($attr_array);
}

// Show a "Change implementation status" picker.
// QQQ
function change_implementation_status($id)
{
  $connection = connect_mysql();

  $request_array = get_dataowner_request_record($id);

  if (!array_key_exists('request_id', $request_array)) {
    throw new Exception("did not find a result for request id '$id'");
  }

  if (session_status() !== PHP_SESSION_ACTIVE) {
    session_start();
  }

  $implementation_status = $request_array['implementation_status'];

  $statuses = array("PENDING", "IMPLEMENTED", "ABANDONED");
  $options = "";
  foreach ($statuses as $cur_status) {
    if ($implementation_status == $cur_status) {
      $selected = ' selected';
    } else {
      $selected = '';
    }
    $options .= "<option value=\"$cur_status\"$selected>$cur_status</option>";
  }

  $str = <<<EOD
<input type="hidden" name="request_id" id="request_id" value="$id">
<input type="hidden" name="current_implementation_status"
       id="current_implementation_status" value="$implementation_status">
<select id="change_implementation_status_select" name="change_implementation_status_select">
$options
</select>
EOD;

  return $str;

}

function update_implementation_status($request_id, $implementation_status) {
  $connection = connect_mysql();

  $sql = "
    UPDATE dataowner_request
    SET    implementation_status = '$implementation_status'
    WHERE  request_id = '$request_id'";

  $result = $connection->query($sql);
  $connection->close();

  if (session_status() !== PHP_SESSION_ACTIVE) {
    session_start();
  }
  $_SESSION["message"] = 'The record for ' . $request_id . ' has been updated';
  $_SESSION["message_type"] = 'SUCCESS';
  header('Location: dataowner_list_requests.php');
}

function update_access_principal($request_id, $access_principal) {
  $connection = connect_mysql();

  $sql = "
    UPDATE dataowner_request
    SET    access_principal = '$access_principal'
    WHERE  request_id = '$request_id'";

  $result = $connection->query($sql);
  $connection->close();

  if (session_status() !== PHP_SESSION_ACTIVE) {
    session_start();
  }
  $_SESSION["message"] = 'The record for ' . $request_id . ' has been updated';
  $_SESSION["message_type"] = 'SUCCESS';
  header('Location: dataowner_list_requests.php');
}

function change_access_principal($id)
{
  $connection = connect_mysql();

  $request_array = get_requests($id, false) ;

  if (!array_key_exists($id, $request_array)) {
    throw new Exception("did not find a result for request id '$id'");
  }

  if (session_status() !== PHP_SESSION_ACTIVE) {
    session_start();
  }

  // Step 1. Get the request details from the database.
  $request = $request_array[$id];

  $request_id              = $request['request_id'];
  $requestor_uid           = $request['requestor_uid'];
  $requestor_name          = $request['requestor_name'];
  $request_type            = $request['request_type'];
  $access_principal        = $request['access_principal'];

  $input_id = "change-access-principal-input";

  $str = <<<EOD
<input type="hidden" name="request_id" id="request_id" value="$id">
<strong>Principal name:</strong>
<input type="text" value="$access_principal" name="$input_id" id ="$input_id"/>
EOD;

  return $str;
}


// Show the details for request with id $id
//
// Note that the details shown will vary depending on the
// the role of the currently logged-in user.
function get_request_details($id)
{
  $connection = connect_mysql();

  $request_array = get_requests($id, false) ;

  if (!array_key_exists($id, $request_array)) {
    throw new Exception("did not find a result for request id '$id'");
  }

  if (session_status() !== PHP_SESSION_ACTIVE) {
    session_start();
  }

  // Step 1. Get the request details from the database.
  $request = $request_array[$id];

  $request_id              = $request['request_id'];
  $requestor_uid           = $request['requestor_uid'];
  $requestor_name          = $request['requestor_name'];
  $request_type            = $request['request_type'];
  $application_description = $request['application_description'];
  $comment                 = $request['comment'];
  $access_principal        = $request['access_principal'];
  $requested_date          = $request['requested_date'];
  $status                  = $request['status'];
  $implementation_status   = $request['implementation_status'];
  $attribute               = $request['attribute'];
  $attributes              = $request['attributes'];
  $attribute_type          = $request['attribute_type'];

  if (empty($application_description)) {
    $application_description = 'No application description supplied.';
  } else {
    // Convert new lines to <br>'s.
    $application_description = nl2br(trim($application_description));
  }

  if (empty($comment)) {
    $comment = 'No extra comments.';
  } else {
    $comment = nl2br(trim($comment));
  }

  if (!empty($requestor_name)) {
    $requestor_name_formatted = "$requestor_uid ($requestor_name)";
  } else {
    $requestor_name_formatted = "$requestor_uid";
  }

  switch ($request_type) {
    case 'ENTITYID':
      $request_type_formatted = 'SAML IdP';
      $identifier_type        = 'Entity ID';
      break;
    case 'KPRINCIPAL':
      $request_type_formatted = 'LDAP Access';
      $identifier_type        = 'Kerberos principal';
      break;
    default:
      $request_type_formatted = 'Unknown request type';
      $identifier_type        = 'Access identifier';
      break;
  }

  if ($attribute_type == 'BUNDLE') {
    $attribute_title = "Bundle: $attribute";
    $attribute_list = get_attribute_list_from_group($attribute);
    $attributes_formatted = join(', ', $attribute_list) ;
  } else {
    $attribute_title = 'Attributes';
    $attributes_formatted = join(', ', $attributes);
  }

  // Step 2. Get the request approval details.
  // Date reviewed will be date_last_maint
  // We show ALL approvers, even those that are not active. Note
  // that the approval pull-downs are shows for all approvers
  // ACTIVE AT THE TIME OF REQUEST. If an approver is added or removed
  // AFTER the request was made, the request's approvers DO NOT CHANGE.
  $sql = "
    SELECT a.approver_uid                   as uid,
           ifnull(a.approver_name, '')      as full_name,
           ifnull(a.status, '')             as status,
           ifnull(a.email_status, '')       as email_status,
           ifnull(a.comment, '')            as comment,
           ifnull(a.date_last_maint, '')    as date_last_maint,
           a.role                           as role
    FROM   dataowner_approval a
           WHERE (a.request_id = $id) AND (a.role = 'APPROVE')
    ORDER BY role DESC, full_name";

  $result = $connection->query($sql);
  while ($row = $result->fetch_assoc()) {
    $attr_array[] = $row ;
  }

  // Now get the current admins. We add a pull-down for implementation
  // status ONLY if the overall status is approved.
  $sql = "
    SELECT a.approver_uid                   as uid,
           ifnull(a.approver_name, '')      as full_name,
           ifnull(a.status, '')             as status,
           ifnull(a.email_status, '')       as email_status,
           ifnull(a.comment, '')            as comment,
           ifnull(a.date_last_maint, '')    as date_last_maint,
           a.role                           as role
    FROM   dataowner_approval a
           WHERE (a.request_id = $id) AND (a.role = 'ADMINISTRATOR')
    ORDER BY full_name";

  $result = $connection->query($sql);
  while ($row = $result->fetch_assoc()) {
    $attr_array[] = $row ;
  }

  $connection->close();

  $str = <<<EOD
<input type="hidden" name="request_id" id="request_id" value="$id">
<strong>Access request type:</strong> $request_type_formatted<br />
<strong>$identifier_type:</strong> <code>$access_principal</code><br />
<strong>Requested by:</strong> $requestor_name_formatted<br />
<strong><a href="javascript:request_attributes1('attributes_$id');">$attribute_title
  (click to show/hide)</a>
  <p style="display: none;" id="attributes_$id">$attributes_formatted</p>
<h3>Application Description</h3>
$application_description
<h3>Comments</h3>$comment
EOD;


  $str = $str . '<h3>Approval History</h3>';
  $str = $str . '<table>
    <tr>
      <th>Approver</th>
      <th>Status</th>
      <th>Comment</th>
      <th>Date Reviewed</th>
    </tr>';

  $username = $_SERVER['REMOTE_USER'];

  if (is_admin()) {
    $admin = True;
  } else {
    $admin = False;
  }

  if (is_approver()) {
    $approver = True;
  } else {
    $approver = False;
  }

  // Loop over all approvers and admins
  $ct = 0;
  if (isset($attr_array)) {
    foreach ($attr_array as $val) {

      // Get the day and time (YYYY-MM-DD HH:MM)
      if (!$val["date_last_maint"] OR ($val["date_last_maint"] == '0000-00-00 00:00:00')) {
        $date_reviewed = '';
      } else {
        $cdate = explode(' ', $val["date_last_maint"]);
        $day  = $cdate[0];
        $time = $cdate[1];
        preg_match('/^(\d\d:\d\d)/', $time, $matches);
        $timeHHMM = $matches[1];
        $date_reviewed = "$day $timeHHMM";
      }

      $str = $str . '<tr>';

      switch ($val['role']) {
        case 'APPROVE':
          $role = '';
          break;
        case 'ADMINISTRATOR':
          $role = '<br \>(admin)';
          break;
        default:
          $role = '';
          break;

      }

      $full_name = $val['full_name'];

      $str = $str . '<td>' . $full_name . $role . '</td>';

      if ($admin == True || ($username == $val["uid"] && $approver) ) {
        //
        // build select list
        //
        $picker = build_pick_list($val["role"],
                                  $ct,
                                  $val["status"],
                                  $val["comment"],
                                  $val["uid"]
                  );
        $str .= $picker;
        $ct += 1;
      } else {
        if (preg_match('/approv/i', $val['role'])) {
          $str = $str . '<td style="max-width:100%">' . $val["status"] . '</td>';
        } else {
          $str = $str . '<td style="max-width:100%"></td>';
        }
        $str = $str . '<td style="white-space: normal !important">' . $val["comment"] . '</td>';
      }
      $str = $str . '<td>' . $date_reviewed . '</td>';
      $str = $str . '</tr>';
    }
  }

  $str = $str . '</table>';
  return ($str);
}

function build_pick_list($role, $ct, $status, $comment, $uid) {
  $result = '';

  // $picker is the picker for the approvers
  $picker  = '';
  $picker .= '<select class="selectpicker" id="new_status' . $ct . '" name="new_status'
             . $ct . '" style="width: 120px">';

  if ($status == 'PENDING' || strlen($status) < 2) {
    $picker = $picker . '<option selected>PENDING</option>';
  } else {
    $picker = $picker . '<option>PENDING</option>';
  }

  if ($status == 'APPROVED') {
    $picker = $picker . '<option selected>APPROVED</option>';
  } else {
    $picker = $picker . '<option>APPROVED</option>';
  }

  if ($status == 'REJECTED') {
    $picker = $picker . '<option selected>REJECTED</option>';
  } else {
    $picker = $picker . '<option>REJECTED</option>';
  }

  if ($status == 'WAITING') {
    $picker = $picker . '<option selected>WAITING</option>';
  } else {
    $picker = $picker . '<option>WAITING</option>';
  }

  $picker = $picker . '</select>';

  if (preg_match('/approv/i', $role)) {
    $result .= '<td style="max-width:100%">' . $picker . '</td>';
  } else {
    $result .= '<td style="max-width:100%"></td>';
  }

  //
  // add comments and original values
  //
  $td_str = <<<EOD
<td style="white-space: normal !important">
<textarea id="comments$ct" name="comments$ct" style="margin-top:2px;width:350px;"
          onmouseover="textAreaAdjust(this)"
     onkeyup="textAreaAdjust(this)">$comment</textarea>
<input type="hidden" id="orig_vals$ct" name="orig_vals$ct" value="$uid|$status|$comment">
</td>
EOD;

  $result .= $td_str ;

  return $result;
}

function format_requester_name($uid, $name)
{
  if (empty($name)) {
    if (empty($uid)) {
      return 'unknown';
    } else {
      return $uid;
    }
  } else {
    if (empty($uid)) {
     return $name;
    } else {
     return "$uid ($name)";
    }
  }
}

function get_request_history($id)
{
  $connection = connect_mysql();

  $sql = "
    SELECT user_uid,
           status,
           email_status,
           date_added,
           comment
    FROM   dataowner_approval
    WHERE  request_id = " . $id;

  $connection->set_charset('utf8');
  $result = $connection->query($sql);
  if (!$result) {
    die('Query execution error: ' . mysql_error());
  }

  $str = '<table>';
  while ($row = $result->fetch_assoc()) {
    $str = $str . '<tr>';
    $str = $str . '<td>' . $row["user_uid"] . '</td>';
    $str = $str . '<td>' . $row["status"] . '</td>';
    $str = $str . '<td>' . $row["email_status"] . '</td>';
    $str = $str . '<td>' . $row["date_added"] . '</td>';
    $str = $str . '<td>' . $row["comment"] . '</td>';
    $str = $str . '<tr>';
  }
  $str = $str . '</table>';
  $connection->close();
  return ($str);
}

/*
  Returns a hash of requests keyed to the request ids. Under normal
  circumstances this means a hash of length one with a single key equal to $id.

  Each request contains all the information from the data_request table along
  with a special field "attribute" which is a string with the attributes concatenated
  together separated by "<br>"'s.
*/
function get_requests($id, $all = True)
{
  $connection = connect_mysql();

  if ($all == True) {
    $operator = ' >= ';
    $and_limit_to_nondeleted = "AND (dataowner_request.status != 'DELETED')";
  } else {
    $operator = ' = ';
    $and_limit_to_nondeleted = '';
  }

  if (session_status() !== PHP_SESSION_ACTIVE) {
    session_start();
  }

  if (isset($_SESSION["status"])) {
    $status_filter = strtoupper($_SESSION["status"]);
  } else {
    $status_filter = '';
  }

  if (!isset($_SESSION['menu_option']))
  {
    //If not isset -> set with dummy value
     $_SESSION['menu_option'] = "";
  }

  if ((is_admin() || is_approver() || is_auditor())
     && ($_SESSION["menu_option"] != "user")) {
    $username = $_SERVER['REMOTE_USER'];

    if (is_approver() || is_auditor()) {
      # APPROVER
      $sql = "
        SELECT dataowner_request.request_id,
               dataowner_request.requestor_uid,
               dataowner_request.requestor_name,
               dataowner_request.request_type,
               dataowner_request.application_description,
               dataowner_request.comment,
               dataowner_request.access_principal,
               dataowner_request.requested_date,
               dataowner_request.status,
               dataowner_request.implementation_status,
               dataowner_request_attr.attribute,
               dataowner_request_attr.attribute_type,
                    dataowner_approval.status AS this_approver_status
        FROM   dataowner_request
        LEFT OUTER JOIN
               dataowner_request_attr ON
               dataowner_request.request_id = dataowner_request_attr.request_id
        JOIN
               dataowner_approval ON
               (dataowner_request_attr.request_id = dataowner_approval.request_id)
               AND (dataowner_approval.approver_uid = '$username')
        WHERE
               dataowner_request.request_id $operator $id and dataowner_request.status like '%$status_filter%' $and_limit_to_nondeleted
        ORDER BY
           1, 6";
    } else {
      # ADMIN
      $sql = "
        SELECT dataowner_request.request_id,
               dataowner_request.requestor_uid,
               dataowner_request.requestor_name,
               dataowner_request.request_type,
               dataowner_request.application_description,
               dataowner_request.comment,
               dataowner_request.access_principal,
               dataowner_request.requested_date,
               dataowner_request.status,
               dataowner_request.implementation_status,
               dataowner_request_attr.attribute,
               dataowner_request_attr.attribute_type
        FROM   dataowner_request
        LEFT OUTER JOIN
               dataowner_request_attr ON
               dataowner_request.request_id = dataowner_request_attr.request_id
        WHERE
               dataowner_request.request_id $operator $id and dataowner_request.status like '%$status_filter%' $and_limit_to_nondeleted
        ORDER BY
           1, 6";
    }
  } else {
    # REGULAR USER
    $username = $_SERVER['REMOTE_USER'];
    $sql = "
      SELECT dataowner_request.request_id,
             dataowner_request.requestor_uid,
             dataowner_request.requestor_name,
             dataowner_request.request_type,
             dataowner_request.application_description,
             dataowner_request.comment,
             dataowner_request.access_principal,
             dataowner_request.requested_date,
             dataowner_request.status,
             dataowner_request.implementation_status,
             dataowner_request_attr.attribute,
             dataowner_request_attr.attribute_type
      FROM   dataowner_request
      LEFT OUTER JOIN
             dataowner_request_attr ON
             dataowner_request.request_id = dataowner_request_attr.request_id
      WHERE
             dataowner_request.request_id $operator $id and
             requestor_uid = '$username' $and_limit_to_nondeleted
      ORDER BY
         1, 5";
  }

  $connection->set_charset('utf8');
  $result = $connection->query($sql);
  if (!$result) {
    die('Query execution error: ' . $connection->error);
  }

  $req_id           = 0;
  $request_array    = [];
  $attributes_array = [];
  $num_requests     = 0;
  while ($row = $result->fetch_assoc()) {
    ++$num_requests;
    // We (might) get multiple rows for a given request id corresponding
    // to multiple attributes.

    // We skip any attributes with the name "CUSTOM". These are legacy
    // attributes.
    if ($row["attribute"] == 'CUSTOM') {
      continue ;
    }

    array_push($attributes_array, $row["attribute"]);
    if ($row["request_id"] > $req_id) {
      // This is a NEW request.
      $req_id                 = $row["request_id"];
      $request_array[$req_id] = $row;
    } elseif ($req_id > 0) {
      // We have seen this request before.
      $request_array[$req_id]["attribute"] = $request_array[$req_id]["attribute"] . ';' . $row["attribute"];
    }
  }

  if (($num_requests > 0) and ($id > 0)) {
    $request_array[$id]["attributes"] = array_unique($attributes_array);
  }

  $connection->close();

  return ($request_array);
}

function get_dataowner_request_record($id)
{
  $connection = connect_mysql();

  $sql = "
      SELECT dataowner_request.request_id,
             dataowner_request.requestor_uid,
             dataowner_request.requestor_name,
             dataowner_request.request_type,
             dataowner_request.application_description,
             dataowner_request.comment,
             dataowner_request.access_principal,
             dataowner_request.requested_date,
             dataowner_request.status,
             dataowner_request.implementation_status
        FROM dataowner_request
        WHERE
             dataowner_request.request_id = $id";

  $connection->set_charset('utf8');
  $result = $connection->query($sql);
  if (!$result) {
    die('Query execution error: ' . $connection->error);
  }

  while ($row = $result->fetch_assoc()) {
    return $row;
  }
}

function get_users()
{
  $connection = connect_mysql();

  $sql = "
    SELECT uid,
           full_name,
           title,
           email,
           role,
           status
    FROM   dataowner_reviewer
    ORDER BY
           1";

  $user_array = Array();
  $connection->set_charset('utf8');
  $result = $connection->query($sql);
  if (!$result) {
    die('Query execution error: ' . $connection->error);
  }

  $ct = 0;
  while ($row = $result->fetch_assoc()) {
    $user_array[$ct] = $row;
    $ct++;
  }
  $connection->close();
  return ($user_array);
}

function get_user_detail($username)
{
  $connection = connect_mysql();

  $sql = "
    SELECT uid,
           full_name,
           title,
           email,
           role,
           status
    FROM   dataowner_reviewer
    WHERE  uid = '$username'
    ORDER BY
           1";

  $user_array = Array();
  $connection->set_charset('utf8');
  $result = $connection->query($sql);
  if (!$result) {
    die('Query execution error: ' . mysql_error());
  }

  $ct = 0;
  while ($row = $result->fetch_assoc()) {
    $user_array[$ct] = $row;
    $ct++;
  }
  $connection->close();
  return ($user_array);
}

function get_dataowner_sequence()
{
  $connection = connect_mysql();

  $sql = "SELECT nextval('dataowner-request')";
  $result = $connection->query($sql);
  if (! $result) {
    die("no nextval result!?!");
  } else {
    $row = $result->fetch_assoc();
  }

  $seq = $row["nextval('dataowner-request')"];
  $connection->close();
  return $seq;
}

function insert_new_user($insRec)
{
  $connection = connect_mysql();
  //
  // mysql> desc dataowner_reviewer;
  // +-----------------+---------------------------+------+-----+---------+-------+
  // | Field           | Type                      | Null | Key | Default | Extra |
  // +-----------------+---------------------------+------+-----+---------+-------+
  // | uid             | varchar(32)               | NO   | PRI | NULL    |       |
  // | full_name       | varchar(64)               | NO   |     | NULL    |       |
  // | title           | varchar(64)               | NO   |     | NULL    |       |
  // | email           | varchar(255)              | YES  |     | NULL    |       |
  // | role            | enum('APPROVE','AUDIT')   | NO   |     | NULL    |       |
  // | status          | enum('ACTIVE','INACTIVE') | NO   |     | ACTIVE  |       |
  // | date_last_maint | datetime                  | NO   |     | NULL    |       |
  // | date_added      | datetime                  | NO   |     | NULL    |       |
  // +-----------------+---------------------------+------+-----+---------+-------+
  //
  $sql = "
    SELECT uid
    FROM   dataowner_reviewer
    WHERE  uid = '$insRec[0]'";

  $result = $connection->query($sql);
  if ($result->num_rows > 0) {
    if (session_status() !== PHP_SESSION_ACTIVE) {
      session_start();
    }
    $_SESSION["message"] = 'The user ' . $insRec[0] . ' already exists';
    $_SESSION["message_type"] = 'DANGER';
    header('Location: dataowner_edit_users.php');
    exit;
  }

  $sql = "
    INSERT into dataowner_reviewer(
      uid,
      full_name,
      title,
      email,
      role,
      status,
      date_last_maint,
      date_added) VALUES (
      '$insRec[0]',
      '$insRec[1]',
      '$insRec[2]',
      '$insRec[3]',
      '$insRec[4]',
      'ACTIVE',
      '$insRec[5]',
      '$insRec[5]')";
  $result = $connection->query($sql);
  $connection->close();

  if (session_status() !== PHP_SESSION_ACTIVE) {
    session_start();
  }
  $_SESSION["message_type"] = 'SUCCESS';
  $_SESSION["message"] = 'The user ' . $insRec[0] . ' has been created with ' . $insRec[4] . ' privileges';
  header('Location: dataowner_edit_users.php');
}

function update_user($updateRec)
{
  $connection = connect_mysql();

  $sql = "
    UPDATE dataowner_reviewer
    SET    email           = '$updateRec[1]',
           full_name       = '$updateRec[2]',
           role            = '$updateRec[3]',
           status          = '$updateRec[4]',
           title           = '$updateRec[5]',
           date_last_maint = '$updateRec[6]'
    WHERE  uid = '$updateRec[0]'";

  $result = $connection->query($sql);
  $connection->close();

  if (session_status() !== PHP_SESSION_ACTIVE) {
    session_start();
  }
  $_SESSION["message"] = 'The record for ' . $updateRec[0] . ' has been updated';
  $_SESSION["message_type"] = 'SUCCESS';
  header('Location: dataowner_edit_users.php');
}

function update_dataowner_request($request_id, $request_type, $value)
{
  $connection = connect_mysql();

  if ($request_type == "STATUS") {
    $sql = "
      UPDATE dataowner_request
      SET    status           = '$value'
      WHERE  request_id = '$request_id'";
  } else {
    $sql = "
      UPDATE dataowner_request
      SET    implementation_status = '$value'
      WHERE  request_id = '$request_id'";
  }
  $result = $connection->query($sql);
  $connection->close();

  return $result;
}

function delete_user($updateRec)
{
  $connection = connect_mysql();

  $sql = "
    DELETE
    FROM   dataowner_reviewer
    WHERE  uid = '$updateRec[0]'";

  $result = $connection->query($sql);
  $connection->close();

  if (session_status() !== PHP_SESSION_ACTIVE) {
    session_start();
  }
  $_SESSION["message"] = 'The user ' . $updateRec[0] . ' has been deleted';
  $_SESSION["message_type"] = 'SUCCESS';
  header('Location: dataowner_edit_users.php');
}

/*
  This function creates a new dataowner_request record as well
  as one dataowner_approval record for each approver.
*/
function insert_dataowner_request($insRec, $attribute_type, $attrRec)
{
  $connection = connect_mysql();

  $request_id     = $insRec[0];
  $requestor_uid  = $connection->real_escape_string($insRec[1]);
  $fullname       = $connection->real_escape_string($insRec[2]);
  $request_type   = $connection->real_escape_string($insRec[3]);
  $principal      = $connection->real_escape_string($insRec[4]);
  $description    = $connection->real_escape_string($insRec[5]);
  $comments       = $connection->real_escape_string($insRec[6]);
  $requested_date = $insRec[7] ;

  $date_last_maint = date("Y-m-d H:i:s");

  $sql = "
    INSERT into dataowner_request(
      request_id,
      requestor_uid,
      requestor_name,
      request_type,
      access_principal,
      application_description,
      comment,
      requested_date,
      status,
      implementation_status,
      date_last_maint,
      date_added) VALUES (
       $request_id,
      '$requestor_uid',
      '$fullname',
      '$request_type',
      '$principal',
      '$description',
      '$comments',
      '$requested_date',
      'PENDING',
      'PENDING',
      '$date_last_maint',
      '$requested_date')";

  $result = $connection->query($sql);
  if (!$result) {
    throw new Exception("error with SQL INSERT: " . $connection->error);
  }

  foreach ($attrRec as $item) {
    $sql = "
      INSERT into dataowner_request_attr(
        request_id,
        attribute,
        attribute_type,
        date_last_maint,
        date_added) VALUES (
        $request_id,
        '$item',
        '$attribute_type',
        '$requested_date',
        '$requested_date')";
    $result = $connection->query($sql);
  }

  // Create approval records, one for each Approver and Administrator.
  // The record for Adminstrator is not for approval, rather for adding
  // comments.
  $sql = 'SELECT * FROM dataowner_reviewer ';
  $sql .= "WHERE status = 'ACTIVE' ";
  $result = $connection->query($sql);

  # Store the reviewer records
  while ($row = $result->fetch_assoc()) {
    $reviewer_info = array(
                      'uid'       => $row['uid'],
                      'full_name' => $row['full_name'],
                      'title'     => $row['title'],
                      'role'      => $row['role'],
                                        );
    $reviewers[$row['uid']] = $reviewer_info;
  }

  $approve_sql = 'INSERT INTO dataowner_approval (';
  $approve_sql .= 'request_id, ';
  $approve_sql .= 'approver_uid, ';
  $approve_sql .= 'approver_name, ';
  $approve_sql .= 'approver_title, ';
  $approve_sql .= 'role, ';
  $approve_sql .= 'date_last_maint, ';
  $approve_sql .= 'date_added ';
  $approve_sql .= ') VALUES (?, ?, ?, ?, ?, ?, ?) ';
  $stmt = $connection->prepare($approve_sql);

  $today = date("Y-m-d H:i:s");

  foreach ($reviewers as $reviewer_uid => $reviewer_info) {
    $full_name = $reviewer_info['full_name'];
    $title     = $reviewer_info['title'];
    $role      = $reviewer_info['role'];
    $stmt->bind_param("issssss", $request_id, $reviewer_uid, $full_name, $title, $role, $today, $today);
    $stmt->execute();
  }


  $connection->close();
  //
  //
  //
  header('Location: dataowner_list_requests.php?request_id=' . $request_id);
}

function get_user_information($username)
{
  $connection = connect_mysql();

  //
  // $username = $_SERVER['REMOTE_USER'];
  //
  // mysql> desc dataowner_reviewer;
  // +-----------------+---------------------------+------+-----+---------+-------+
  // | Field           | Type                      | Null | Key | Default | Extra |
  // +-----------------+---------------------------+------+-----+---------+-------+
  // | uid             | varchar(32)               | NO   | PRI | NULL    |       |
  // | full_name       | varchar(64)               | NO   |     | NULL    |       |
  // | title           | varchar(64)               | NO   |     | NULL    |       |
  // | email           | varchar(255)              | YES  |     | NULL    |       |
  // | role            | enum('APPROVE','AUDIT')   | NO   |     | NULL    |       |
  // | status          | enum('ACTIVE','INACTIVE') | NO   |     | ACTIVE  |       |
  // | date_last_maint | datetime                  | NO   |     | NULL    |       |
  // | date_added      | datetime                  | NO   |     | NULL    |       |
  // +-----------------+---------------------------+------+-----+---------+-------+

  $sql = "
    SELECT uid,
           role,
           status,
           email,
           full_name,
           title,
           date_added,
           date_last_maint
    FROM   dataowner_reviewer
    WHERE  uid = '$username'";

  $result = $connection->query($sql);
  $row    = $result->fetch_assoc();
  $connection->close();

  return $row;
}

function check_resp()
{
  $username   = $_SERVER['REMOTE_USER'];
  $connection = connect_mysql();

  $sql = "
    SELECT role
    FROM   dataowner_reviewer
    WHERE  uid = '$username'";

  $result = $connection->query($sql);
  $row    = $result->fetch_assoc();
  $resp   = $row["role"];

  $connection->close();

  if (session_status() !== PHP_SESSION_ACTIVE) {
    session_start();
  }

  unset($_SESSION["user"]);
  unset($_SESSION["admin"]);
  unset($_SESSION["audit"]);
  unset($_SESSION["approver"]);

  if (preg_match('/ADMIN/', $resp)) {
     $resp = "admin";
  } elseif (preg_match('/APPROV/', $resp)) {
     $resp = "approver";
  } elseif (preg_match('/AUD/', $resp)) {
     $resp = "audit";
  } else {
     $resp = "user";
  }

  if ($resp) {
    $_SESSION[$resp] = True;
  }


  return $resp;
}

function move_requests_to_archive($request_id)
{
  $connection = connect_mysql();

  $sql = "
    INSERT
    INTO   dataowner_request_archive
    SELECT *
    FROM   dataowner_request
    WHERE  request_id <= $request_id";
  $result = $connection->query($sql);
  $ct = $connection->affected_rows;

  $sql = "
    INSERT
    INTO   dataowner_request_attr_archive
    SELECT *
    FROM   dataowner_request_attr
    WHERE  request_id <= $request_id";
  $result = $connection->query($sql);

  $sql = "
    DELETE
    FROM   dataowner_request
    WHERE  request_id <= $request_id";
  $result = $connection->query($sql);

  $sql = "
    DELETE
    FROM   dataowner_request_attr
    WHERE  request_id <= $request_id";
  $result = $connection->query($sql);

  $connection->query("commit");
  $connection->close();

  if (session_status() !== PHP_SESSION_ACTIVE) {
    session_start();
  }
  $_SESSION["message"] = $ct . ' records were archived';
  $_SESSION["message_type"] = 'SUCCESS';
  header('Location: admin_menu.php');
  exit;
}

function soft_delete_request($request_id)
{
  $connection = connect_mysql();
  $sql = "
    UPDATE dataowner_request
    SET    status = 'DELETED'
    WHERE  request_id = $request_id";

  $result = $connection->query($sql);
  if (!$result) {
      error_log("mysql query error: " . $connection->error);
  }

  $ct = $connection->affected_rows;
  return $ct;
}

function purge_requests_in_archive($request_id)
{
  $connection = connect_mysql();

  $sql = "
    DELETE
    FROM   dataowner_request_archive
    WHERE  request_id <= $request_id";
  $result = $connection->query($sql);
  $ct = $connection->affected_rows;

  $sql = "
    DELETE
    FROM   dataowner_request_attr_archive
    WHERE  request_id <= $request_id";
  $result = $connection->query($sql);

  $connection->query("commit");
  $connection->close();

  if (session_status() !== PHP_SESSION_ACTIVE) {
    session_start();
  }
  $_SESSION["message"] = $ct . ' records were deleted from archive';
  $_SESSION["message_type"] = 'SUCCESS';
  header('Location: admin_menu.php');
  exit;
}

function delete_request($request_id)
{
  $connection = connect_mysql();

  $sql = "
    DELETE
    FROM   dataowner_request
    WHERE  request_id = $request_id";
  $result = $connection->query($sql);
  $ct = $connection->affected_rows;

  $sql = "
    DELETE
    FROM   dataowner_request_attr
    WHERE  request_id = $request_id";
  $result = $connection->query($sql);

  $connection->query("commit");
  $connection->close();

  if (session_status() !== PHP_SESSION_ACTIVE) {
    session_start();
  }
  $_SESSION["message_type"] = 'SUCCESS';
  if ($ct == 1) {
    $_SESSION["message"] = $ct . ' record was deleted';
  } else {
    $_SESSION["message"] = $ct . ' records were deleted';
  }
  header('Location: admin_menu.php');
}

function show_custom_attributes()
{
  $connection = connect_mysql();
  $sql = "
    SELECT *
    FROM   dataowner_attributes
    ORDER BY
           group_name,
           attribute_name";

  $result = $connection->query($sql);
  while ($row = $result->fetch_assoc()) {
    $attr_array[] = $row ;
  }
  $connection->close();
  return ($attr_array);
}

function show_attribute_groups()
{
  $connection = connect_mysql();
  $sql = "
    SELECT *
    FROM   dataowner_attribute_groups
    ORDER BY
           group_name";

  $result = $connection->query($sql);
  while ($row = $result->fetch_assoc()) {
    $attr_array[] = $row ;
  }
  $connection->close();
  return ($attr_array);
}

function remove_attribute($attribute_name, $attribute_group_name)
{
  $connection = connect_mysql();

  $sql = "
    DELETE
    FROM   dataowner_attributes
    WHERE  attribute_name  = '$attribute_name' and
           group_name      = '$attribute_group_name'";

  $result = $connection->query($sql);
  $ct = $connection->affected_rows;
  $connection->close();

  if (session_status() !== PHP_SESSION_ACTIVE) {
    session_start();
  }
  if ($ct > 0) {
    $_SESSION["message"]      =  'The attribute record for ' . $attribute_name . ' in group ' . $attribute_group_name . ' has been deleted';
    $_SESSION["message_type"] = 'SUCCESS';
  } else {
    $_SESSION["message"]      =  'No records deleted';
    $_SESSION["message_type"] = 'DANGER';
  }
  header('Location: admin_menu.php');
}

function remove_attribute_group($attribute_group_name)
{
  $connection = connect_mysql();

  $sql = "
    DELETE
    FROM   dataowner_attributes
    WHERE  group_name = '$attribute_group_name'";

  $result = $connection->query($sql);
  $attr_ct = $connection->affected_rows;

  $sql = "
    DELETE
    FROM   dataowner_attribute_groups
    WHERE  group_name = '$attribute_group_name'";
  $result = $connection->query($sql);
  $ct = $connection->affected_rows;

  $connection->close();

  if (session_status() !== PHP_SESSION_ACTIVE) {
    session_start();
  }
  if ($ct > 0) {
    $_SESSION["message"] =  'Attribute group ' . $attribute_group_name . ' has been deleted and ' . $attr_ct . ' attribute records';
    $_SESSION["message_type"] = 'SUCCESS';
  } else {
    $_SESSION["message"] =  'No records deleted';
    $_SESSION["message_type"] = 'DANGER';
  }
  header('Location: admin_menu.php');
}

function new_attribute($attribute_string, $attribute_group_name)
{

  if (!isset($attribute_name)) {
        $attribute_name = '';
  }

  $connection = connect_mysql();
  $attr_name = $connection->real_escape_string($attribute_name);
  #
  # $group_name = mysql_real_escape_string($attribute_group_name);
  #
  $group_name = $connection->real_escape_string($attribute_group_name);

  $attributes = preg_split('/[     ,\n\r]/', $attribute_string, 0, PREG_SPLIT_NO_EMPTY | PREG_SPLIT_DELIM_CAPTURE);
  $exists_ct = 0;
  $insert_ct = 0;

  foreach ($attributes as $attribute_name) {
    $sql = "
      SELECT attribute_name
      FROM   dataowner_attributes
      WHERE  attribute_name = '$attribute_name' and
             group_name = '$group_name'";

    $result = $connection->query($sql);
    #$ct = $connection->affected_rows();
    $ct = $connection->affected_rows;

    if ($ct > 0) {
      $exists_ct += 1;
    } else {
      $username = $_SERVER['REMOTE_USER'];
      $date = new DateTime();
      $current_date = $date->format('Y-m-d H:i:s');
      $sql = "
        INSERT
        INTO   dataowner_attributes (
          attribute_name,
          group_name,
          created_by_uid,
          created_date)
        VALUES (
          '$attribute_name',
          '$group_name',
          '$username',
          '$current_date'
        )";

      $result = $connection->query($sql);
      $ct = $connection->affected_rows;

      if ($ct > 0) {
        $insert_ct += 1;
      }
    }
  }

  if (session_status() !== PHP_SESSION_ACTIVE) {
    session_start();
  }
  if ($exists_ct > 0 && $insert_ct == 0) {
    $_SESSION["message"] =  $exists_ct . ' attributes failed to load. Attributes already exist';
    $_SESSION["message_type"] = 'DANGER';
  } else {
    $_SESSION["message_type"] = 'SUCCESS';
    if ($insert_ct > 0 && $exists_ct == 0) {
      $_SESSION["message"] =  $insert_ct . ' attributes have been loaded';
    } else {
      $_SESSION["message"] =  $insert_ct . ' attributes have been loaded. ' . $exists_ct . ' already exist';
    }
  }
  $connection->close();
  header('Location: admin_menu.php');
}

function new_attribute_group($attribute_group_name)
{
  $connection = connect_mysql();
  $group_name = $connection->real_escape_string($attribute_group_name);

  $sql = "
    SELECT group_name
    FROM   dataowner_attribute_groups
    WHERE  group_name = '$group_name'";

  $result = $connection->query($sql);
  $ct = $connection->affected_rows;

  if (session_status() !== PHP_SESSION_ACTIVE) {
    session_start();
  }

  if ($ct > 0) {
    $_SESSION["message"] =  'The attribute group ' . $group_name . ' already exists';
    $_SESSION["message_type"] = 'DANGER';
  } else {
    $username = $_SERVER['REMOTE_USER'];
    $date = new DateTime();
    $current_date = $date->format('Y-m-d H:i:s');
    $sql = "
      INSERT
      INTO   dataowner_attribute_groups (
        group_name,
        created_by_uid,
        created_date)
      VALUES (
        '$group_name',
        '$username',
        '$current_date'
      )";
    $result = $connection->query($sql);
    $ct = $connection->affected_rows;
    if ($ct > 0) {
      $_SESSION["message"] =  'The attribute group ' . $group_name . ' has been added';
      $_SESSION["message_type"] = 'SUCCESS';
    }
  }
  $connection->close();

  header('Location: admin_menu.php');
}

function update_request_approval($request_id, $approver_uid, $status, $comment)
{
  //
  // mysql> desc dataowner_approval
  // +-----------------+-------------------------------------------------+------+-----+---------+-------+
  // | Field           | Type                                            | Null | Key | Default | Extra |
  // +-----------------+-------------------------------------------------+------+-----+---------+-------+
  // | request_id      | int(11)                                         | NO   | PRI | NULL    |       |
  // | approver_uid    | varchar(32)                                     | NO   | PRI | NULL    |       |
  // | approver_name   | varchar(64)                                     | NO   |     | NULL    |       |
  // | approver_title  | varchar(64)                                     | NO   |     | NULL    |       |
  // | role            | enum('APPROVE','AUDIT')                         | NO   |     | APPROVE |       |
  // | status          | enum('PENDING','APPROVED','REJECTED','WAITING') | NO   |     | PENDING |       |
  // | email_status    | enum('PENDING','SENT','RESEND')                 | NO   |     | PENDING |       |
  // | comment         | text                                            | YES  |     | NULL    |       |
  // | date_last_maint | datetime                                        | NO   |     | NULL    |       |
  // | date_added      | datetime                                        | NO   |     | NULL    |       |
  // +-----------------+-------------------------------------------------+------+-----+---------+-------+
  //
  $connection = connect_mysql();

  $sql = "
    SELECT full_name,
           title,
           role
    FROM   dataowner_reviewer
    WHERE  uid = '$approver_uid'";

  $result = $connection->query($sql);
  $row    = $result->fetch_assoc();

  $full_name = $row["full_name"];
  $title     = $row["title"];
  $role      = $row["role"];

  # Get the current approval record.
  # We only set date_last_maint if the comment or status changes.
  $sql = "
    SELECT comment,
           status
    FROM   dataowner_approval
    WHERE  approver_uid = '$approver_uid' AND (request_id = $request_id)";

  $result = $connection->query($sql);
  $row    = $result->fetch_assoc();

  $current_status  = $row["status"];
  $current_comment = $row["comment"];

  $date         = new DateTime();
  $current_date = $date->format('Y-m-d H:i:s');
  $comment_str  = $connection->real_escape_string($comment);

  # Only update status if it set and not "undefined".
  $set = "
    SET    comment       = '$comment_str',
           approver_name   = '$full_name',
           approver_title  = '$title',
           role            = '$role'";

  if (($status == 'undefined') or (!isset($status)) or empty($status)) {
    $status = NULL;
  }

  if (($current_comment != $comment_str)
     || (!($current_status === $status))) {
     $set = $set . ", date_last_maint = '$current_date'" ;
  }

  if (!is_null($status)) {
    $set = $set . ", status = '$status'";
  }

  $sql = "
    UPDATE dataowner_approval
    $set
    WHERE  request_id = $request_id and
           approver_uid = '$approver_uid'";

  $result = $connection->query($sql);
  if (!$result) {
    error_log($connection->error);
  }

  $ct = $connection->affected_rows;

  if (session_status() !== PHP_SESSION_ACTIVE) {
    session_start();
  }
  if ($ct == 0) {
    $sql = "
      INSERT
      INTO   dataowner_approval(
        request_id,
        approver_uid,
        approver_name,
        approver_title,
        role,
        status,
        comment,
        date_last_maint,
        date_added) VALUES (
        $request_id,
        '$approver_uid',
        '$full_name',
        '$title',
        '$role',
        '$status',
        '$comment',
        '$current_date',
        '$current_date')";
    $result = $connection->query($sql);
    $_SESSION["message_type"] = 'SUCCESS';
    $_SESSION["message"] = $_SESSION["message"] . 'Request ID ' . $request_id . ' approval status for ' . $approver_uid . ' has been inserted';
  } else {
    $_SESSION["message_type"] = 'SUCCESS';

    if (empty($_SESSION["message"])) {
      $cur_message = '';
    } else {
      $cur_message = $_SESSION["message"];
    }
    $_SESSION["message"] = $cur_message . 'Request ID ' . $request_id . ' approval status for ' . $approver_uid . ' has been updated';

    $subject = "Data Access Request $request_id has been updated by $approver_uid";
    $role_formatted = ucfirst(strtolower($role));

    if (preg_match("/approv/i", $role)) {
      $role_formatted = 'Approver';
    } else if (preg_match("/admin/i", $role)) {
      $role_formatted = 'Administrator';
    } else if (preg_match("/audi/i", $role)) {
      $role_formatted = 'Auditor';
    } else {
      $role_formatted = 'Role-Unknown';
    }

    if (($status == 'undefined') or (!isset($status)) or empty($status)) {
      $status = 'N/A';
    }

    $body = <<<EOT
$role_formatted $approver_uid has updated their status and/or comment for this ticket. As an ADMINISTRATOR it is your job to communicate to the requester any issues this comment may generate.

$approver_uid's status:
$status

$approver_uid's comment:
$comment
EOT;
    if (preg_match('/APPROVED/i', $status)) {
      // Do nothing.
    } else {
      mail_admins($subject, $body);
    }
  }
  $connection->close();
}

function mail_admins($subject, $body)
{
  $connection = connect_mysql();

  # Get a list of all the admin information.
  $sql = "
  SELECT full_name,
         email
  FROM   dataowner_reviewer
  WHERE    role = 'ADMINISTRATOR'
     AND status = 'ACTIVE'";

  $from = 'noreply@stanford.edu';

  $result = $connection->query($sql);
  while ($row = $result->fetch_assoc()) {
    $full_name = $row["full_name"];
    $email     = $row["email"];

    $to = $email;
    if (!$body) {
      $body = "Empty message.";
    }
    send_mail($to, $from, $subject, $body);
  }

  $connection->close();
}

function send_mail($to, $from, $subject, $body) {
  ini_set("SMTP", "smtp-unencrypted.stanford.edu");
  ini_set("sendmail_from", $from);

  # Get the email override (if it is set).
  load_config();
  $configuration =  $GLOBALS['CONFIG'];

  if (!preg_match("/true/i", $configuration['production_mode'])) {
    $subject = "[TESTING MODE] $subject";
  }

  if ($configuration['email_override']) {
    $prepend = <<<EOT
[NOTE: all recipient addresses have been overridden. Here are the original e-mail addresses: $to]
EOT;
    $body = $prepend . "\n\n" . $body;
    $to = $configuration['email_override'];
  }

  $headers = "From: $from" . "\r\n" .
    "Reply-To: $from" . "\r\n" .
    'X-Mailer: PHP/' . phpversion();

  mail($to, $subject, $body, $headers);

}

# ------------------------------------------------------------------------
# dataowner status check.  Check the status of the approver rows and
# set the status of the request if there is one rejection or if all
# approvers have approved the change.

function do_check_status($request_id)
{
    $connection = connect_mysql();

    # Set the overall status if necessary
    $sql  = 'SELECT status, count(*) cnt FROM dataowner_approval ';
    $sql .= "WHERE request_id = $request_id ";
    $sql .= "AND role = 'APPROVE' ";
    $sql .= 'GROUP BY status ';

    $result = $connection->query($sql);
    if (!$result) {
      $msg = 'Query execution error: ' . mysql_error();
      error_log($msg);
      die($msg);
    }

    $approved    = 0;
    $tot         = 0;
    $update_flag = 0;
    while ($row = $result->fetch_assoc()) {
        if (($row['status'] == 'REJECTED') || ($row['status'] == 'WAITING')) {
            # The veto status values.  Just set the request to this
            # status if any of the approvers have set it.
            do_set_status($row['status'], $request_id);
            $update_flag = 1;
            break;
        } elseif ($row['status'] == 'APPROVED') {
            $approved += $row['cnt'];
        }
        $tot += $row['cnt'];
    }

    # If there are only approvals then set the request status to APPROVED
    if (!$update_flag && ($approved == $tot)) {
        do_set_status('APPROVED', $request_id);
        $update_flag = 1;
    }

    # If the status has not been set yet, the status is still pending.
    if (!$update_flag) {
      do_set_status('PENDING', $request_id);
    }

    return;
}

# ----------------------------------------------------------
# dataowner status set
# Set the status for a request

function do_set_status($status, $request_id)
{
    $connection = connect_mysql();

    # Get previous status value
    $sql  = 'SELECT status FROM dataowner_request ';
    $sql .= "WHERE request_id = $request_id ";

    $result = $connection->query($sql);
    if (!$result) {
      $msg = 'Query execution error: ' . mysql_error();
      error_log($msg);
      die($msg);
    }

    $old = '';
    if ($row = $result->fetch_assoc()) {
        $old = $row['status'];
    } else {
        # print some error here.
        $msg = 'Did not find a status: ' . mysql_error();
        error_log($msg);
        die($msg);
    }

    if ($old == $status) {
      return;
    }

    # Update the status now that we know it has changed
    $a_now = sql_datetime();

    $sql = 'UPDATE dataowner_request SET ';
    $sql .= "status = '$status', ";
    $sql .= "date_last_maint = '$a_now' ";
    $sql .= "WHERE request_id = $request_id ";

    $result = $connection->query($sql);
    $ct = $connection->affected_rows;

    if ($ct == 0) {
      die("query failed: " . mysql_error());
    }

    $log_entry = array(
      'updateID'  => 'SETSTATUS',
      'action'    => 'UPDATE',
      'table'     => 'dataowner_request',
      'keyFields' => 'request_id',
      'keyValues' => $request_id,
      'fieldName' => 'status',
      'oldValue'  => $old,
      'newValue'  => $status,
      'uid'       => $_SERVER['REMOTE_USER'],
      );
    do_log_change($log_entry);

    $msg = "Request Status for Request ID  $request_id set to $status<br>\n";
    return $msg;
}

function do_log_change($parm)
{
    $r = 'do_log_change';
    $id = next_number('DATAOWNER-MAINT');

    $connection = connect_mysql();

    $cmd  = "INSERT INTO dataowner_log ";
    $cmd .= "SET ";
    $cmd .= " record_id  = ?, ";
    $cmd .= " update_id  = ?, ";
    $cmd .= " action     = ?, ";
    $cmd .= " table_name = ?, ";
    $cmd .= " key_fields = ?, ";
    $cmd .= " key_values = ?, ";
    $cmd .= " field_name = ?, ";
    $cmd .= " old_value  = ?, ";
    $cmd .= " new_value  = ?, ";
    $cmd .= " uid        = ?, ";
    $cmd .= " date_added = NOW() ";

    $stmt = $connection->prepare($cmd);

    $stmt->bind_param("isssssssss",
                      $id,
                      $parm['updateID'],
                      $parm['action'],
                      $parm['table'],
                      $parm['keyFields'],
                      $parm['keyValues'],
                      $parm['fieldName'],
                      $parm['oldValue'],
                      $parm['newValue'],
                      $parm['uid']
                      );

    if (!$stmt->execute()) {
      $msg = "log update failed" . mysql_error();
      error_log($msg);
      die($msg);
    }

    return;
}

# ----------------------------------------------------------
# get unique number

function next_number($a_id)
{
  if (!$a_id) {
    throw new Exception("Invalid ID for next_number");
  }

  $connection = connect_mysql();

  # Look to see if we have one at all
  $sel    = "SELECT next_number FROM next_number WHERE id='$a_id' ";
  $result = $connection->query($sel);
  $row    = $result->fetch_assoc();

  $new_nn = 0;
  if ($row['next_number']) {
      # Increment the next_number (which is really the last number used)
      $cmd  = "UPDATE next_number SET ";
      $cmd .= "next_number = LAST_INSERT_ID(next_number+1), ";
      $cmd .= "date_last_maint = NOW() ";
      $cmd .= "WHERE id='$a_id' ";
      $result = $connection->query($cmd);

      # Retreive the resulting value
      $sel    = "SELECT LAST_INSERT_ID() next_number ";
      $result = $connection->query($sel);
      $nnrow  = $result->fetch_assoc();
      $new_nn = $nnrow['next_number'];
  } else {
      # Create a new one
      $new_nn = 1;

      $cmd  = "INSERT INTO next_number SET ";
      $cmd .= "id='$a_id', ";
      $cmd .= "next_number = $new_nn, ";
      $cmd .= "date_last_maint = NOW(), ";
      $cmd .= "date_added = NOW() ";
      $result = $connection->query($cmd);
  }
  return $new_nn;
}

function sql_datetime()
{
  $date = new DateTime();
  $current_date = $date->format('Y-m-d H:i:s');

  return $current_date;
}


#function process_new_request()
#{
#  $connection = connect_mysql();
#
#  $sql = "
#    SELECT request_id
#    FROM   dataowner_request
#    WHERE  status = 'SUBMITTED'";
#
#  $result = $connection->query($sql);
#  if (!$result) {
#    die('Query execution error: ' . mysql_error());
#  }
#  while ($row = $result->fetch_assoc()) {
#    $requests[] = $row;
#  }
#
#  //
#  // print_r($requests);
#  //
#
#  $date = new DateTime();
#  $current_date = $date->format('Y-m-d H:i:s');
#
#  $reviewers = get_reviewers();
#  foreach ($requests as $req) {
#    $req_id = $req["request_id"];
#    foreach ($reviewers as $r) {
#      $r_uid = $r["uid"];
#      $r_full_name = $r["full_name"];
#      $r_title = $r["title"];
#      $r_role = $r["role"];
#
#      $connection = connect_mysql();
#      $sql = "
#        INSERT
#        INTO   dataowner_approval(
#          request_id,
#          approver_uid,
#          approver_name,
#          approver_title,
#          role,
#          status,
#          comment,
#          date_last_maint,
#          date_added) VALUES (
#          $req_id,
#          '$r_uid',
#          '$r_full_name',
#          '$r_title',
#          '$r_role',
#          'PENDING',
#          '',
#          '$current_date',
#          '$current_date')";
#      //
#      // print $sql;
#      //
#      $result = $connection->query($sql);
#    }
#    $connection = connect_mysql();
#    $sql = "
#      UPDATE dataowner_request
#      SET    status = 'PENDING'
#      WHERE  request_id = $req_id";
#    //
#    // print $sql;
#    //
#    $result = $connection->query($sql);
#  }
#  $connection->close();
#}

?>
