<?php

require_once '_uit_head.php';
require_once '_common.php';
require_once '_db.php';

/**
 * Generate a model <div>..<div> element. Takes as input the div id and title.
*/
function generateModalDiv($id, $title, $show_save_button = true,
                          $button_text = array('Save', 'Close'))
{
  $button_save_text  = $button_text[0];
  $button_close_text = $button_text[1];

  $str = <<<EOM
    <div id="$id-modal" class="modal fade" role="dialog"
         style="width:940px; margin-left: -500px;">
      <div class="modal-dialog" style="width:940px;">
        <div class="modal-content">
          <div class="modal-header">
            <button type="button" class="close" data-dismiss="modal" aria-hidden="true">&times;</button>
            <h3 class="modal-title" id="modal-div-{$id}-title"
                style="color: #0275d8;">$title</h3>
          </div>
          <div class="modal-body" id="modal-div-${id}-body"></div>
          <div class="modal-footer">
            <button type="button" class="btn btn-default"
              id="modal-div-${id}-close" name="modal-div-${id}-close"
              data-dismiss="modal">$button_close_text</button>
EOM;
if ($show_save_button) {
$str .= <<<EOJ
            <button type="button" class="btn btn-default"
              id="modal-div-${id}-save" name="modal-div-${id}-save"
                    data-dismiss="modal">$button_save_text</button>
EOJ;
}
  $str .= <<<EOK
          </div>
        </div>
      </div>
    </div>
EOK;

  return $str;
}

function detailModalDiv($show_save_button = true)
{
  if ($show_save_button) {
    $id = 'detail';
  } else {
    $id = 'detail-nosave';
  }

  // We only show the Save button for approvers and administrators.
  if (!(is_approver() || is_admin())) {
    $show_save_button = false;
  }

  return generateModalDiv($id, 'Request Details', $show_save_button);
}

function infoModalDiv()
{
  return generateModalDiv("info", 'Information', false);
}

function changeImplementationStatusModalDiv()
{
  $id = 'change-impl-status';

  return generateModalDiv($id, 'Set Implementation Status', true);
}

function softDeleteModalDiv()
{
  $id = 'soft-delete';

  return generateModalDiv($id, "Delete Request", true, array('Delete', 'Close'));
}

function changeAccessPrincipalModalDiv()
{
  $id = 'change-access-principal';

  return generateModalDiv($id, 'Change Principal Name', true);
}

function renderForm()
{
  display_header('Stanford Directory Access Request');

  $tab = 'dataowner_list_requests';
  if (isset($_GET["status"])) {
    $_SESSION["status"] = $_GET["status"];
    $tab = 'dataowner_' . $_GET["status"] . '_requests';
  } else {
    $_SESSION["status"] = '';
  }

  nav($tab);

  if (isset($_SESSION["message"])) {
    $message = $_SESSION["message"];
    if (!isset($_SESSION['message_type'])) {
      ///If not isset -> set with dumy value
      $_SESSION['message_type'] = "";
    }
    if ($_SESSION["message_type"] == "DANGER") {
      print '<div class="alert alert-danger"><strong>' . $message . '</strong></div>';
    } else {
      print '<div class="alert alert-success"><strong>' . $message . '</strong></div>';
    }
    unset($_SESSION["message"]);
    unset($_SESSION["message_type"]);
  }

  print <<<EOH
<script type="text/javascript" src="https://code.jquery.com/jquery-1.12.4.js"></script>
<script type="text/javascript" src="javascript/bootstrap/js/bootstrap.min.js"></script>
<script type="text/javascript" src="javascript/jquery-datatables/jquery.dataTables.min.js"></script>
<script type="text/javascript"
        src="javascript/jquery-datatables-extensions/FixedColumns/js/dataTables.fixedColumns.min.js"></script>
<script type="text/javascript"
        src="javascript/jquery-datatables-extensions/Responsive/js/dataTables.responsive.min.js"></script>

<link type="text/css" rel="stylesheet"
      href="javascript/jquery-datatables/css/jquery.dataTables.min.css"/>
<link type="text/css" rel="stylesheet"
      href="javascript/jquery-datatables-extensions/Responsive/css/responsive.bootstrap.min.css"/>

<style>
  .dataTables_filter { text-align: right !important; }
  .dataTables_length select { width: 80px; }
  div.dataTables_length label { width: 200px; float: left; text-align: left; padding-bottom: 10px; }
  div.dataTables_info { padding-top: 10px; }
  div.dataTables_paginate { float: right; margin: 0; }
  div.content-wrapper.content-wrapper { width: 100%; }
  table.dataTable tbody td { vertical-align: top !important; }
  div#content.mc-content.span8 { width: 96%; }
  div.modal-header { width: 886px; }
  div.modal-body { width: 850px; }
  div.modal-footer { width: 886px; }
  div.modal-content { width: 890px; }
}
</style>

<script type="text/javascript">
$(document).ready(function() {
  $('#requests').DataTable({
    pagingType: "full_numbers",
    fixedColumns: true,
    "order": [[ 0, "desc" ]],
    columnDefs: [ { width: 40, targets: 0 } ],
    "aoColumnDefs": [ { "sType": "html-num", "aTargets": [ 0 ] } ],
    scrollX: true
  });
  $('#modal-div-detail-save').on('click', function(e) {
    var str1 = "";
    var str2 = "";
    var str3 = "";
    for (i = 0; i < 99; i++) {
      var selector1 = "#orig_vals" + i;
      var selector2 = "#new_status" + i;
      var selector3 = "#comments" + i;
      if ($(selector1).length) {
        str1 += $(selector1).val() + "^";
        str2 += $(selector2).children("option:selected").val() + "^";
        str3 += $(selector3).val() + "^";
      } else {
        break;
      }
    }
    document.approver_form.f_request_id.value = $('#request_id').val();
    document.approver_form.f_orig_vals.value  = str1;
    document.approver_form.f_new_status.value = str2;
    document.approver_form.f_comments.value   = str3;
    document.approver_form.submit();
  });
  $('#modal-div-change-impl-status-save').on('click', function(e) {
    var status_select_id = "#change_implementation_status_select";
    var selected_option = $(status_select_id).children("option:selected").val();
    document.approver_form.f_impl_status.value = selected_option;
    document.approver_form.f_request_id.value  = $('#request_id').val();
    document.approver_form.submit();
  });
  $('#modal-div-soft-delete-save').on('click', function(e) {
    var request_id = $('#request_id').val();
    //alert('Delete button pushed for request ' + request_id);
    document.approver_form.f_delete_request_id.value = request_id;
    document.approver_form.submit();
  });
  $('#modal-div-soft-delete-close').on('click', function(e) {
    alert('Close button pushed');
    // Do nothing.
  });
  $('#modal-div-change-access-principal-save').on('click', function(e) {
    var access_principal_id = "#change-access-principal-input";
    var access_principal  = $(access_principal_id).val();
    document.approver_form.f_access_principal.value = access_principal;
    document.approver_form.f_request_id.value     = $('#request_id').val();
    document.approver_form.submit();
  });
});

function request_details(id) {
  $.ajax({
    type: 'post',
    url: '_dataowner_list_details.php',
    data: 'param=' + id,
    success: function(d) {
      $('#modal-div-detail-body').html(d);
      $('#modal-div-detail-body').scrollTop(0);
      $('#modal-div-detail-title').html("Approval Details for Request " + id);
    }
  });
  $('#detail-modal').modal();
}

function change_implementation_status(id) {
  $.ajax({
    type: 'post',
    url: '_change_implementation_status.php',
    data: 'param=' + id,
    success: function(d) {
      $('#modal-div-change-impl-status-body').html(d);
      $('#modal-div-change-impl-status-body').scrollTop(0);
      $('#modal-div-change-impl-status-title').html("Change Implementation Status for Request " + id);
    }
  });
  $('#change-impl-status-modal').modal();
}

function change_access_principal(id) {
  $.ajax({
    type: 'post',
    url: '_change_access_principal.php',
    data: 'param=' + id,
    success: function(d) {
      $('#modal-div-change-access-principal-body').html(d);
      $('#modal-div-change-access-principal-body').scrollTop(0);
      $('#modal-div-change-access-principal-title').html("Change Access Principal for Request " + id);
    }
  });
  $('#change-access-principal-modal').modal();
}

function request_attributes(group_name) {
  $.ajax({
    type: 'post',
    url:  '_dataowner_list_attributes.php',
    data: 'param=' + group_name,
    success: function(d) {
      $('#modal-div-info-body').html(d);
      $('#modal-div-info-body').scrollTop(0);
      $('#modal-div-info-title').html("Attributes for " + group_name);
    }
  });
  $('#info-modal').modal();
}

function request_attributes1(attributes_id) {
  $("#" + attributes_id).toggle();
}

function request_history(id) {
  $.ajax({
    type: 'post',
    url: '_dataowner_list_history.php',
    data: 'param=' + id,
    success: function(d) {
      $('.modal-body').html(d);
      $('.modal-body').scrollTop(0);
    }
  });
  $('#detailModal').modal();
}

function soft_delete_request(id) {
  $.ajax({
    type: 'post',
    url: '_soft_delete_request.php',
    data: 'param=' + id,
    success: function(d) {
      $('#modal-div-soft-delete-body').html(d);
      $('#modal-div-soft-delete-body').scrollTop(0);
      $('#modal-div-soft-delete-title').html("Delete Request " + id);
    }
  });
  $('#soft-delete-modal').modal();
}


</script>
EOH;

  $id = 0;
  if (isset($_GET["request_id"])) {
    $id = $_GET["request_id"];
  }
  $requests = get_requests($id);
  unset($_SESSION["status"]);

  if (count($requests) == 0) {
    print "No records available";
    return;
  }

  if ($id > 0) {
    print '<div class="alert alert-success"><strong>Request ID ' . $id . ' Created</strong></div>';
  }
  //
  // bootstrap datatables
  //

  // Define the modals
  $modal_div_with_save_button             = detailModalDiv(true);
  $modal_div_change_implementation_status = changeImplementationStatusModalDiv();
  $modal_div_soft_delete                  = softDeleteModalDiv();
  $modal_div_info                         = infoModalDiv();
  $modal_div_change_access_principal      = changeAccessPrincipalModalDiv();

  $is_approver = is_approver();
  $is_admin    = is_admin();

  if ($is_approver) {
    $approver_status_column      = '<th>Your approval status</th>';
    $overall_approver_status_column = '';
  } else {
    $approver_status_column      = '';
    $overall_approver_status_column = '<th title="Overall approval status">Approval Status</th>';
  }


  print <<<EOL
    <form id="approver_form" name="approver_form" method="post" action="_approver_update_x.php">
      <input type="hidden" id="f_request_id"        name="f_request_id"        value="">
      <input type="hidden" id="f_orig_vals"         name="f_orig_vals"         value="">
      <input type="hidden" id="f_new_status"        name="f_new_status"        value="">
      <input type="hidden" id="f_comments"          name="f_comments"          value="">
      <input type="hidden" id="f_impl_status"       name="f_impl_status"       value="">
      <input type="hidden" id="f_access_principal"  name="f_access_principal"  value="">
      <input type="hidden" id="f_delete_request_id" name="f_delete_request_id" value="">
    </form>

$modal_div_with_save_button

$modal_div_change_implementation_status

$modal_div_soft_delete

$modal_div_info

$modal_div_change_access_principal

    <table id="requests" class="table table-striped table-bordered compact" cellspacing="0" width="100%">
      <thead>
        <tr>
          <th>ID</th>
$approver_status_column
          <th>Requestor</th>
          <th>Requested</th>
          <th>Type</th>
          <th>Principal/EntityID</th>
$overall_approver_status_column
          <th title="Has this request been implemented yet?">Implemented?</th>
          <th>Attributes</th>
          <th></th>
        </tr>
      </thead>
      <tbody>
EOL;

  if (is_array($requests)) {
    // Get a sorted list of the request ids.
    $request_ids = array_keys($requests);
    rsort($request_ids);
    foreach ($request_ids as $request_id) {
      $values          = $requests[$request_id];
      $requestor_name  = format_requester_name($values["requestor_uid"], $values["requestor_name"]);
      $attribute_type  = $values["attribute_type"];

      $approval_status = $values["status"];
      switch ($approval_status) {
        case 'PENDING':
          $approval_status_title = 'This request is waiting for approval';
          break;
        case 'APPROVED':
          $approval_status_title = 'This request has been fully approved';
          break;
        case 'WAITING':
          $approval_status_title = 'At least one approver has paused the approval process';
          break;
        case 'REJECTED':
          $approval_status_title = 'This request has been rejected';
          break;
        case 'DELETED':
          $approval_status_title = 'This request has been DELETED';
          break;
        default:
          $approval_status_title = 'Unknown status';
          break;
      }

      if (array_key_exists("this_approver_status", $values)) {
        $this_approver_status = $values["this_approver_status"];
      } else {
        $this_approver_status = '';
      }

      $date = new DateTime($values["requested_date"]);
      $requested_date = date_format($date, "m/d/Y");

      switch ($values["implementation_status"]) {
        case 'PENDING':
          switch ($approval_status) {
            case 'APPROVED':
              $implementation_status       = 'in progress';
              $implementation_status_title = 'The data access team is working on granting access';
              break;
            case 'REJECTED':
              $implementation_status = 'halted';
              $implementation_status_title = 'Your request has been rejected so will not be implemented';
              break;
            default:
              $implementation_status = 'waiting for all approvals';
              $implementation_status_title = 'Implementation will start once this request is approved by all approvers';
              break;
          }
          break;
        case 'IMPLEMENTED':
          $implementation_status = 'finished';
          $implementation_status_title = 'Your request is approved and implemented';
          break;
        case 'ABANDONED':
          $implementation_status = 'abandoned';
          $implementation_status_title = 'Your request has been abandoned';
          break;
        default:
          $implementation_status = 'not yet';
          break;
      }

      if ($is_admin) {
        $impl_js    = "javascript:change_implementation_status('$request_id')";
        $impl_title = 'Change implementation status';
        $implementation_link = " <a href=\"$impl_js\" title=\"$impl_title\">change</a>";

        $p_change_js    = "javascript:change_access_principal('$request_id')";
        $p_change_title = "Change principal name";
        $p_change_link  = " <a href=\"$p_change_js\" title=\"$p_change_title\">change</a>";
      } else {
        $implementation_link = '';
        $p_change_link       = '';
      }

      print '<tr>';

      // Column 1.
      print '<td>
       <a href="javascript:request_details(\'' . $request_id . '\');"> '
         . $request_id . '</a>
      </td>';


      // Column 1B (only for approvers).
      if ($is_approver) {
        if   (!empty($this_approver_status)
          && (!preg_match('/APPROV|REJE/i', $this_approver_status))) {
          $style = 'color: red; font-weight: bold;';
        } else {
          $style = 'font-weight: normal;';
        }
        print "<td><span style=\"$style\">$this_approver_status</span></td>";
      }

      // Column 2. REQUESTOR
      print "<td>$requestor_name</td>";

      // Column 3. DATE REQUESTED
      $date = new DateTime($values["requested_date"]);
      print "<td>$requested_date</td>";

      $type = $values["request_type"];
      switch ($type) {
          case 'ENTITYID':
            $type_formatted = 'EntityID (SAML)';
            break;
          case 'KPRINCIPAL':
            $type_formatted = 'Kerberos principal';
            break;
           default:
            $type_formatted = 'unknown';
            break;
      }
      // Column 4. TYPE
      print '<td>' . $type_formatted . '</td>';

      $access_principal = strip_tags(trim($values["access_principal"]));
      if (empty($access_principal)) {
        $access_principal = 'not defined';
      } else {

        # Truncate display of long strings
        $max_length = 30;
        if (strlen($access_principal) > $max_length) {
          $access_principal_display = substr($access_principal, 0, $max_length) . '...';
        } else {
          $access_principal_display = $access_principal;
          }
      }

      $access_principal_title = htmlspecialchars($access_principal);

      // Column 4. PRINCIPAL NAME
      print "<td><span title=\"$access_principal_title\" style=\"word-break:break-all;\">$access_principal_display$p_change_link</span></td>";

      // Column 5. APROVAL STATUS
      if (! $is_approver) {
        print "<td title=\"$approval_status_title\">" . strtolower($approval_status) . "</td>";
      }

      // Column 6. IMPLEMENTATION STATUS
      print "<td title=\"$implementation_status_title\">$implementation_status $implementation_link</td>";

      $attributes_array = explode(';', $values["attribute"]);
      if ($values["attribute_type"] == "BUNDLE") {
        if (count($attributes_array) == 1) {
          $attribute = $attributes_array[0];
          print '<td>Bundle: <a href="javascript:request_attributes(\''
           . $attribute . '\');"> ' . $attribute . '</a></td>';
        } else {
          $attributes_formatted   = join("<br />", $attributes_array);
          print "<td>$attributes_formatted</td>";
   }
      } else {
        $attributes_tmp = $values["attribute"];
        $attributes_formatted = str_replace(";", "<br />", $attributes_tmp);
        print "<td>$attributes_formatted</td>";
      }

      // Column ?: (soft) delete request
      if (is_admin()) {
        $delete_js = "javascript:soft_delete_request('$request_id')";
        print '<td><a href="' . $delete_js . '" title="Delete ' . $request_id . '"><img src="trash.png"
               style="max-height: 2.5em; max-width: 2.5em;" width="100%" height="100%"></a></td>';
      } else {
        print "<td></td>";
      }
      print "</tr>\n";
    }
  }

  print '
      </tbody>
    </table>';

  display_footer();
}

/*
function implementationPicker()
{
      switch ($values["implementation_status"]) {
        case 'PENDING':
          switch ($approval_status) {
            case 'APPROVED':
              $implementation_status       = 'in progress';
              $implementation_status_title = 'The data access team is working on granting access';
              break;
            case 'REJECTED':
              $implementation_status = '';
              break;
            default:
              $implementation_status = 'waiting for all approvals';
              $implementation_status_title = 'Implementation will start once this request is approved by all approvers';
              break;
          }
          break;
        case 'IMPLEMENTED':
          $implementation_status = 'finished';
          $implementation_status_title = 'Your request is approved and implemented';
          break;
        default:
          $implementation_status = 'not yet';
          break;
      }

}
*/

renderForm();
