#!/bin/sh

set -e

# Create the file database.ini like this:
#
#OLD_DB_HOST=mysql-app.stanford.edu
#OLD_DB_DBASE=ldaplog
#OLD_DB_USER=ldaplog
#OLD_DB_PASSWORD=secret
#
#NEW_DB_HOST=acs-db.stanford.edu
#NEW_DB_DBASE=dataowner_test
#NEW_DB_USER=dataowner_test
#NEW_DB_PASSWORD=secret

source ./database.ini

OLD_DB_TABLES="dataowner_approval dataowner_log dataowner_request dataowner_request_attr dataowner_reviewer next_number"

NEWTABLES_SQL_FILE=newtables.sql
UPDATE_SCHEMA_SQL_FILE=update-schema.sql
MAKE_ADMINS_SQL_FILE=make-admins.sql
NEXTVAL_SQL_FILE=nextval.sql
POPULATE_ATTRIBUTES_SQL_FILE=populate_attributes.sql

### ### ### ### ### ### ### ### ### ###
# Pass in the .sql file path
new_db_sql_file() {
  mysql -h $NEW_DB_HOST -u $NEW_DB_USER -D $NEW_DB_DBASE \
	--password=$NEW_DB_PASSWORD < $1
}
### ### ### ### ### ### ### ### ### ###

# Step 1. Dump the old database.
echo "dumping old data..."
DUMPFILE=/tmp/olddb.sql
rm -f $DUMPFILE
mysqldump -h $OLD_DB_HOST -u $OLD_DB_USER --password=$OLD_DB_PASSWORD \
  $OLD_DB_DBASE $OLD_DB_TABLES > $DUMPFILE

# Step 2. Drop and create new database.
echo "creating new database..."
SQLFILE=/tmp/createdb.sql
rm -f $SQLFILE
echo "DROP DATABASE IF EXISTS $NEW_DB_DBASE;" > $SQLFILE
echo "CREATE DATABASE $NEW_DB_DBASE;"         >> $SQLFILE
mysql -h $NEW_DB_HOST -u $NEW_DB_USER --password=$NEW_DB_PASSWORD \
  < $SQLFILE
rm -f $SQLFILE

# Step 3. Create the nextval function
echo "creating the nextval function..."
new_db_sql_file $NEXTVAL_SQL_FILE


# Step 4. Load old data into new database.
echo "loading old data into new database..."
new_db_sql_file $DUMPFILE

# Step 5. Add new tables
echo "adding new tables..."
new_db_sql_file $NEWTABLES_SQL_FILE

# Step 6. Update the schema of some of the old tables.
echo "updating schema of old tables..."
new_db_sql_file $UPDATE_SCHEMA_SQL_FILE

# Step 7. Populate the attribute groups and attributes
echo "populating the attribute groups and attributes..."
new_db_sql_file $POPULATE_ATTRIBUTES_SQL_FILE

# Step 8. Make some people admins.
echo "make some users admins..."
new_db_sql_file $MAKE_ADMINS_SQL_FILE
